#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

#define serialPort
#define BTN_UP   1
#define BTN_DOWN 2
#define BTN_LEFT 3
#define BTN_RIGHT 4
#define BTN_SELECT 5
#define BTN_NONE 10

// стоковые параметры для таймера
#define Motor_ESC_Max_THR 2200      // максимальные обороты двигателя
#define Motor_ESC_Normall_THR 2000  // номинальные (полетные) обороты
#define Motor_ESC_Landing_THR 1400  // посадочный режим
#define Motor_ESC_Min_THR 800       // двигатель стоп
#define Time_Of_Calibrate 2         // время между Макс газ и мин газ при необходимости калибровке регулятора
#define Takeoff 120                 // время отведенное на взлет и уборку шасси, в течении этого времени процедура старта может быть отменена (т.е. время до уборки шасси) -  10 секунд 
#define GearUp 40                   // время время после старта, начало уборки шасси -  5 секунд 
#define InFly 330                   // время полета в секундах с момента завершения процедуры старта - сейчас стоит 60 секунд
#define Landing 40                  // время отводимое на посадку, в течении этого времени будут выпущены шасси и снижены обороты мотора до посадочных -  60 секунд 
#define delay_g 100                 // коэффициент торможения ))
#define free1 0                 
#define free2 0                 
#define free3 0                 
#define free4 0                 
#define free5 0                 
#define free6 0                 
#define free7 0                 
#define free8 0                 


uint8_t stateButtonKey = 0;
int countMenu = 15;
boolean flagButtonState = true;                              // изменилось состояние кнопок
uint8_t lastStateButton = 10;                                 // нет нажатия кнопок
uint64_t timeOfChange;
uint8_t position = 0;

int8_t data [14] [4];   // массив для отображения побайтно

char* menu [17] {
  "THR Norm Level  ",     // нормальные обороты мотора
  "THR Land Level  ",     // обороты мотора при посадке
  "FREE-1 uint16_t ",     // 1 резервный пункт меню числовой параметр
  "FREE-2 uint16_t ",     // 2 резервный пункт меню числовой параметр
  "FREE-3 uint16_t ",     // 3 резервный пункт меню числовой параметр
  "Flight time     ",     // общее время полета с момента уборки шасси
  "Time for start  ",     // время на "убежать к ручке"
  "Time for GUp    ",     // момент уборки шасси с момента старта
  "Time for landing",     // время на посадку.
  "FREE-4 uint16_t ",     // 4 резервный пункт меню временной параметр
  "FREE-5 uint16_t ",     // 5 резервный пункт меню временной параметр
  "FREE-6 uint16_t ",     // 6 резервный пункт меню временной параметр
  "FREE-1 bollean  ",     // 1 резервный пункт меню логический параметр
  "FREE-2 bollean  ",     // 2 резервный пункт меню логический параметр
  "Write data      ",     // записать данные в таймер
  "Read data       "      // прочитать данные с таймера
};

uint16_t Stack [] {            // программирумые параметры
  Motor_ESC_Normall_THR,  // нормальные обороты мотора
  Motor_ESC_Landing_THR,  // обороты мотора при посадке
  free1,
  free2,
  free3,
  InFly,                  // общее время полета с момента уборки шасси
  Takeoff,                // время на "убежать к ручке"
  GearUp,                 // момент уборки шасси с момента старта
  Landing,                // время на посадку.
  free4,
  free5,
  free6,
  free7,
  free8
};

int readButton() 
{
  int keyAnalog =  analogRead(A0);
    if (keyAnalog < 50) return BTN_RIGHT; 
    else if (keyAnalog < 150) return BTN_UP;
    else if (keyAnalog < 300) return BTN_DOWN;
    else if (keyAnalog < 550) return BTN_LEFT;
    else if (keyAnalog < 750) return BTN_SELECT;
    else return BTN_NONE;
}
/*
void clearLine(int line)
{
  lcd.setCursor(line, 1);
  lcd.print("                ");
}
*/
void printDisplay(uint8_t pos, uint8_t line, String message)
{
  lcd.setCursor(pos, line);
  lcd.print(message);
}
//*******************************************************************************
void readData()
{
  uint8_t pos = 0;
  uint8_t i = 0;
  printDisplay(6, 1, "Read OK!");
  Serial.println("Read OK!");
  for (i = 0; i<13; i++)
  {
    if (i >= 0 && i <= 4) pos = 0;
    else if (i >= 5 && i <= 11) pos = 1;
    else pos = 2;
  Serial.println(i);
  Serial.println(Stack[i]);
    switch (pos)
    {
      case 0:
      data[i][0] = Stack[i]/1000;
      data[i][1] = (Stack[i]-data[i][0]*1000)/100;
      data[i][2] = (Stack[i]-data[i][0]*1000-data[i][1]*100)/10;
      data[i][3] = (Stack[i]-data[i][0]*1000-data[i][1]*100-data[i][2]*10);
      break;
      case 1:
      data[i][0] = Stack[i]/60;                     // минуты
      data[i][3] = (Stack[i]-data[i][0]*60)/10;     // десятки секунд
      break;
    }
  }

}
void writeData()
{
  printDisplay(0, 1, "write timer data");

}
//*******************************************************************************
void setup() 
{
#ifdef serialPort
  Serial.begin(9600);
#endif
  lcd.begin(16, 2);
  lcd.print("Programmer ready");
  lcd.cursor();
//  lcd.blink();

#ifdef serialPort
  Serial.println("Programmer ready");
#endif
printDisplay(0,0,menu[countMenu]);
}

void loop() 
{
  stateButtonKey = readButton();
  if ((stateButtonKey != lastStateButton) && flagButtonState) 
  {
    Serial.println("Button");
    Serial.println(stateButtonKey);
    flagButtonState = false;      // изменилось состояние и флаг поднят
    timeOfChange = millis();
  }
    if (millis() - timeOfChange > 120  && (stateButtonKey != lastStateButton)) 
    {
      Serial.println("Wait");
      lastStateButton = stateButtonKey;
      Serial.println(stateButtonKey);
      flagButtonState = true;
      switch (stateButtonKey) 
      {
//***********************************************************************
        case BTN_UP:
        if (countMenu >= 0 && countMenu <= 4) 
        {
          data [countMenu][position]++;
          if (data [countMenu][position] > 9) data [countMenu][position] = 0;
          lcd.setCursor(position, 1);
          lcd.print(data [countMenu][position]);
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 5 && countMenu <= 11) 
        {
          data [countMenu][position]++;
          if (position == 0 && data [countMenu][position] > 30) data [countMenu][position] = 0;
          if (position == 3 && data [countMenu][position] > 5) data [countMenu][position] = 0;
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 min:sec");
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          if (data [countMenu][0] == 0) data [countMenu][0] = 1; else data [countMenu][0] = 0;
          lcd.setCursor(0, 1);
          if (data [countMenu][0] == 0) lcd.print("False       "); else lcd.print("True        "); 
          lcd.setCursor(2, 1);
        }
        break;
//*********************************************************************
        case BTN_DOWN:
        if (countMenu >= 0 && countMenu <= 4) 
        {
          data [countMenu][position]--;
          if (data [countMenu][position] < 0) data [countMenu][position] = 9;
          lcd.setCursor(position, 1);
          lcd.print(data [countMenu][position]);
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 5 && countMenu <= 11) 
        {
          data [countMenu][position]--;
          if (position == 0 && data [countMenu][position] < 0) data [countMenu][position] = 30;
          if (position == 3 && data [countMenu][position] < 0) data [countMenu][position] = 5;
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 min:sec");
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          if (data [countMenu][0] == 0) data [countMenu][0] = 1; else data [countMenu][0] = 0;
          lcd.setCursor(0, 1);
          if (data [countMenu][0] == 0) lcd.print("False       "); else lcd.print("True        "); 
          lcd.setCursor(2, 1);
        }
        break;
//*********************************************************************
        case BTN_LEFT:
        countMenu--;
        if (countMenu < 0) countMenu = 15;
        printDisplay(0,0,menu[countMenu]);
        if (countMenu >= 0 && countMenu <= 4)
        {
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.print(data[countMenu][1]);
          lcd.print(data[countMenu][2]);
          lcd.print(data[countMenu][3]);
          lcd.setCursor(0, 1);
        }
        if (countMenu >= 5 && countMenu <= 11)
        {
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 min:sec");
          lcd.setCursor(2, 1);
        }
        if (countMenu >= 12 && countMenu <= 13)
        {
          if (data [countMenu][0] == 0) 
          {
            printDisplay(0,1,"               ");
            printDisplay(0,1,"False          ");
          }
          else
          {
            printDisplay(0,1,"               ");
            printDisplay(0,1,"True           ");
          }
        }
        if (countMenu >= 14 && countMenu <= 15)
        {
        printDisplay(0,1,"               ");
        }
        position = 0;
        lcd.setCursor(position, 1);
        break;
//**************************************************************        
        case BTN_RIGHT:
        countMenu++;
        if (countMenu > 15) countMenu = 0;
        printDisplay(0,0,menu[countMenu]);
        if (countMenu >= 0 && countMenu <= 4)
        {
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.print(data[countMenu][1]);
          lcd.print(data[countMenu][2]);
          lcd.print(data[countMenu][3]);
          lcd.setCursor(0, 1);
        }
        if (countMenu >= 5 && countMenu <= 11)
        {
          printDisplay(0,1,"               ");
          lcd.setCursor(0, 1);
          lcd.print(data[countMenu][0]);
          lcd.setCursor(2, 1);
          lcd.print(":");
          lcd.print(data[countMenu][3]);
          lcd.print("0 min:sec");
          lcd.setCursor(2, 1);
        }
        if (countMenu >= 12 && countMenu <= 13)
        {
          if (data [countMenu][0] == 0) 
          {
            printDisplay(0,1,"               ");
            printDisplay(0,1,"False          ");
          }
          else
          {
            printDisplay(0,1,"               ");
            printDisplay(0,1,"True           ");
          }
        }
        if (countMenu >= 14 && countMenu <= 15)
        {
          printDisplay(0,1,"               ");
        }
        position = 0;
        lcd.setCursor(position, 1);
        break;
//****************************************************************************
        case BTN_SELECT:
        if (countMenu == 14) {writeData(); break;}
        if (countMenu == 15) {readData(); break;}
        if (countMenu >= 0 && countMenu <=4) 
        {
          position++;
          if (position > 3) position = 0;
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 5 && countMenu <= 11) 
        {
          if (position == 0) position = 3; else position = 0;
          lcd.setCursor(position, 1);
        }
        if (countMenu >= 12 && countMenu <= 13) 
        {
          position = 0;
          lcd.setCursor(position, 1);
        }
        break;
      }
    }
}
